#ifndef __VGL_H
#define __VGL_H
/*
Header for
V-Tech Genius Leader 4000

for SDCC compiler

2019-06-27 by Bernhard "HotKey" Slawik
*/

#define byte unsigned char
#define word unsigned short

volatile __at (0x0000) byte VGL_MEM[0x10000];

volatile __at (0x0000) byte VGL_BANK0[0x4000];	// Switched via port 0x00
volatile __at (0x4000) byte VGL_BANK1[0x4000];	// Switched via port 0x01
volatile __at (0x8000) byte VGL_BANK2[0x4000];	// Switched via port 0x03 - also: CARTRIDGE if 0x80 set
volatile __at (0xc000) byte VGL_RAM[0x2000];

/*
// MODEL2000
volatile __at (0xdce0) byte KEY_STATUS;	// Controls reading from the keyboard on 2000 (put 0xc0 into it, wait for it to become 0xd0)
volatile __at (0xdce4) byte KEY_CURRENT;	// Holds the current key code on 2000
*/

// MODEL4000
volatile __at (0xdb00) byte KEY_STATUS;	// Controls reading from the keyboard on 4000 (put 0xc0 into it, wait for it to become 0xd0)
volatile __at (0xdb01) byte KEY_CURRENT;	// Holds the current key code on 4000

/*
ports:
	0x00	bank0	0x0000 - 0x3FFF
	0x01	bank1	0x4000 - 0x7FFF
	0x03	bank2	0x8000 - 0xBFFF
	
	0x0a	LCD
	0x0b	LCD
	
	0x10	printer data
	0x11	printer flags (BUSY)
	0x12	system flags:
				0x01=Power off system?
				0x02=?
				0x04=printer STROBE
				0x08=?
				0x10=?
				0x20=Caps lock LED
				0x40=?
				0x80=?
*/

// Key codes
#define KEY_NONE	0
#define KEY_ASTERISK	0x2a	// Asterisk (top left key) without modifiers
#define KEY_BREAK	0x60	// Asterisk (top left key) + Shift

#define KEY_HELP	0x83	// HELP/TAB without modifiers
#define KEY_TAB		0x82	// HELP/TAB + Shift

#define KEY_PLAYER_LEFT	0x7b	// LEFT PLAYER without modifiers
#define KEY_PLAYER_RIGHT	0x7f	// RIGHT PLAYER without modifiers
//#define KEY_PLAYER_LEFT	0x5b	// LEFT PLAYER + Shift
//#define KEY_PLAYER_RIGHT	0x5f	// RIGHT PLAYER + Shift
//#define KEY_INS	0x7b
//#define KEY_DEL	0x7f

#define KEY_CURSOR_RIGHT	0xf0
#define KEY_CURSOR_LEFT	0xf1
#define KEY_CURSOR_UP	0xf2	// uUml + Alt
#define KEY_CURSOR_DOWN	0xf3	// aUml + Alt
#define KEY_BACKSPACE_X	0xf4	// oUml + Alt

#define KEY_ANSWER	0x2b	// +/ANSWER (does not change with Shift)
#define KEY_ENTER	0x7c

#define KEY_REPEAT	0x7e	// REPEAT/DELETE without modifiers
#define KEY_DELETE	0x5C	// REPEAT/DELETE + Shift

#define KEY_ACTIVITY_1	0x01	// First activity selection foil button
#define KEY_ACTIVITY_2	0x02
#define KEY_ACTIVITY_3	0x03
#define KEY_ACTIVITY_4	0x04
#define KEY_ACTIVITY_5	0x05
#define KEY_ACTIVITY_6	0x06
#define KEY_ACTIVITY_7	0x07
#define KEY_ACTIVITY_8	0x08
#define KEY_ACTIVITY_9	0x09
#define KEY_ACTIVITY_10	0x0a
#define KEY_ACTIVITY_11	0x0b
#define KEY_ACTIVITY_12	0x0c
#define KEY_ACTIVITY_13	0x0d
#define KEY_ACTIVITY_14	0x0e
#define KEY_ACTIVITY_15	0x0f
#define KEY_ACTIVITY_16	0x10
#define KEY_ACTIVITY_17	0x11
#define KEY_ACTIVITY_18	0x12
#define KEY_ACTIVITY_19	0x13
#define KEY_ACTIVITY_20	0x14
#define KEY_ACTIVITY_21	0x15
#define KEY_ACTIVITY_22	0x16
#define KEY_ACTIVITY_23	0x17
#define KEY_ACTIVITY_24	0x18
#define KEY_ACTIVITY_25	0x19
#define KEY_ACTIVITY_26	0x1a
#define KEY_ACTIVITY_27	0x1b
#define KEY_ACTIVITY_28	0x1c
#define KEY_ACTIVITY_29	0x1d
#define KEY_ACTIVITY_30	0x1e
#define KEY_ACTIVITY_31	0x1f	// This is the last one they could fit below valid ASCII codes
#define KEY_ACTIVITY_32	0xB0	// This is where they continue (0xb0)
#define KEY_ACTIVITY_33	0xB1
#define KEY_ACTIVITY_34	0xB2
#define KEY_ACTIVITY_35	0xB3	// Last activity selection foil button

#define KEY_POWER_OFF	0xB4	// OFF foil button
#define KEY_LEVEL	0xB5	// LEVEL foil button
#define KEY_PLAYERS	0xB6	// PLAYERS foil button
#define KEY_DISKETTE	0xB7	// DISKETTE foil button

void vgl_halt() __naked {
// As seen in MODEL4000 ROM 0181
__asm
	ld	a, #0x01
	out	(#0x0a), a
	
	ld	a, #0x01
	out	(#0x12), a
	halt
__endasm;
}

/*
void vgl_reboot() {
__asm
	ld	a, #0x00
	out	(#0x03), a
__endasm;
}


void vgl_switchBank0(byte a) {
	(void)a;
__asm
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	
	out	(#0x00), a
__endasm;
}
void vgl_switchBank1(byte a) {
	(void)a;
__asm
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	
	out	(#0x01), a
__endasm;
}
void vgl_switchBank2(byte a) {
	(void)a;
__asm
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	
	out	(#0x03), a
__endasm;
}
void vgl_switchBank(byte n, byte a) {
	switch(n) {
		case 0: vgl_switchBank0(a); break;
		case 1: vgl_switchBank1(a); break;
		case 2: vgl_switchBank2(a); break;
	}
}


void delay(word cycles) {
	(void)cycles;	// Suppress warning "unreferenced function argument"
	
__asm
	push	hl
	push	af
	
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l, a
	
	_delay_loop:
		dec	hl
		ld	a,h
		or	l
		jr	nz, _delay_loop
	pop	af
	pop	hl
__endasm;
}
*/


#include "vgl_sound.h"
void beep() {
	vgl_sound_note(12*4 + 0, 250);
	//vgl_sound_note(12*4 + 0, 50);
}


// Mandatory stdio defines
char getchar() {
	byte c;
	
	KEY_STATUS = 0xc0;
	while (KEY_STATUS != 0xd0) {}
	c = KEY_CURRENT;
	
	switch(c) {
		case KEY_BACKSPACE_X:	return 0x08;	// Map to backspace
		case KEY_DELETE:	return 0x08;	// Map to backspace
		case KEY_REPEAT:	return 0x08;	// Delete: Map to backspace
		
		case KEY_HELP:	return 0x09;	// TAB
		case KEY_TAB:	return 0x09;	// TAB
		
		case KEY_ENTER:	return 0x0a;	// Map to new line (to work with gets() in MAME)
		
	}
	
	return c;
}

#include "vgl_lcd.h"
void putchar(char c) {
	switch(c) {
	case 0x07:	// BELL
		beep();
		break;
	case 0x08:	// BKSP
		if (vgl_cursorX > 0) vgl_cursorX--;
		vgl_lcd_update();
		break;
	case 0x09:	// TAB
		vgl_cursorX += 4;
		vgl_lcd_update();
		break;
	case 0x0a:	// LF
		vgl_cursorX = 0;
		vgl_cursorY ++;
		vgl_lcd_update();
		break;
	case 0x0d:	// CR
		vgl_cursorX = 0;
		vgl_lcd_update();
		break;

	default:
		vgl_lcd_putchar(c);
	}
}

void clear() {
	vgl_lcd_clear();
}

#endif //__VGL_H