#ifndef __VGL_LCD_H
#define __VGL_LCD_H
/*
LCD code for V-Tech Genius Leader 1000/2000/400x
2019-07-02 Bernhard "HotKey" Slawik

For more info about the LCD see MAME code:
	https://github.com/mamedev/mame/blob/master/src/devices/video/hd44780.cpp
	https://github.com/mamedev/mame/blob/master/src/.../pc2000.cpp

*/



/*
TODO:
	Implement proper "lcd_busy()" check as described in:
	http://www.8051projects.net/lcd-interfacing/commands.php
	
	
*/

//#define byte unsigned char
//#define word unsigned short


/*
// MODEL2000
#define LCD_COLS 20
#define LCD_ROWS 2
volatile __at (0xdca0) byte LCD_VRAM[20*2];
volatile __at (0xdca0) byte LCD_VRAM_ROW0[20];	// ... 0xdcb3
volatile __at (0xdcb4) byte LCD_VRAM_ROW1[20];	// ... 0xdcc7

volatile __at (0xdceb) byte LCD_REFRESH_ROWS[2];
volatile __at (0xdceb) byte LCD_REFRESH_ROW0;	// MODEL2000: Refreshes display of VRAM_ROW0 when 1 is put there
volatile __at (0xdcec) byte LCD_REFRESH_ROW1;	// MODEL2000: Refreshes display of VRAM_ROW1 when 1 is put there
volatile __at (0xdced) byte LCD_CURSOR_MODE;	// Show cursor (0=off, 1=block 2=line)
volatile __at (0xdcef) byte LCD_CURSOR_COL;	// Current column + 64*row
*/

// MODEL4000
#define LCD_COLS 20
#define LCD_ROWS 4
volatile __at (0xdca0) byte LCD_VRAM[20*4];
volatile __at (0xdca0) byte LCD_VRAM_ROW0[20];	// ... 0xdcb3
volatile __at (0xdcb4) byte LCD_VRAM_ROW1[20];	// ... 0xdcc7
volatile __at (0xdcc8) byte LCD_VRAM_ROW2[20];	// ... 0xdcdb	// MODEL4000 only (MODEL2000 stores cursor data there)
volatile __at (0xdcdc) byte LCD_VRAM_ROW3[20];	// ... 0xdcef	// MODEL4000 only (MODEL2000 stores cursor data there)

volatile __at (0xdcf0) byte LCD_REFRESH_ROWS[4];
volatile __at (0xdcf0) byte LCD_REFRESH_ROW0;	// MODEL4000: Refreshes display of VRAM_ROW0 when 0 is put there
volatile __at (0xdcf1) byte LCD_REFRESH_ROW1;	// MODEL4000: Refreshes display of VRAM_ROW0 when 1 is put there
volatile __at (0xdcf2) byte LCD_REFRESH_ROW2;	// MODEL4000: Refreshes display of VRAM_ROW0 when 2 is put there
volatile __at (0xdcf3) byte LCD_REFRESH_ROW3;	// MODEL4000: Refreshes display of VRAM_ROW0 when 3 is put there
volatile __at (0xdcf4) byte LCD_CURSOR_COL;
volatile __at (0xdcf5) byte LCD_CURSOR_ROW;
volatile __at (0xdcf6) byte LCD_CURSOR_MODE;	// Show cursor (0=off, 1=block 2=line)


byte vgl_cursorX = 0;
byte vgl_cursorY = 0;

void vgl_delay_1fff() {
__asm
	; Used for screen functions (after putting stuff to ports 0x0a or 0x0b)
	push	hl
	ld	hl, #0x1fff
_delay_1fff_loop:
	dec	l
	jr	nz, _delay_1fff_loop
	dec	h
	jr	nz, _delay_1fff_loop
	pop	hl
__endasm;
}


void vgl_delay_010f() {
__asm
	; Used for screen functions (after putting stuff to ports 0x0a or 0x0b)
	push	hl
	ld	hl, #0x010f
_delay_010f_loop:
	dec	l
	jr	nz, _delay_010f_loop
	dec	h
	jr	nz, _delay_010f_loop
	pop	hl
__endasm;
}



void vgl_lcd_writeControl(byte a) {	// aka "_out_0x0a"
	(void)a;	// suppress warning "unreferenced function argument"
	
__asm
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	
	; Put it to port
	out	(0x0a), a
	
	;call _delay_010f
__endasm;
	vgl_delay_010f();
	vgl_delay_010f();
}

void vgl_lcd_writeData(byte a) {	// aka "_out_0x0b"
	(void)a;	// suppress warning "unreferenced function argument"
	
__asm
	; Get parameter from stack into a
	ld hl,#0x0002
	add hl,sp
	ld a,(hl)
	
	; Put it to port
	out	(0x0b), a
	
	;call _delay_1fff
__endasm;
	vgl_delay_1fff();
}

void vgl_lcd_clear(void);
void vgl_lcd_update(void);


void vgl_lcd_init() {
	// This performs the LCD initialization
	// as seen in several ROMs and in the HD44780 data sheet itself
	
	// Enter 8 bit mode
	vgl_lcd_writeControl(0x38);	//Function set: 2 Line, 8-bit, 5x7 dots
	vgl_lcd_writeControl(0x38);
	vgl_lcd_writeControl(0x38);
	vgl_lcd_writeControl(0x38);
	
	// CLS
	vgl_lcd_writeControl(0x01);
	
	
	// bit 3 (0x08): display on/off control (0=blink, 1=cursor, 2=display)
	vgl_lcd_writeControl(0x0f);
	//vgl_lcd_writeControl(0x0e);
	//vgl_lcd_writeControl(0x0c);
	
	// bit 2 (0x04): entry mode set (0=direction, 1=shift)
	//vgl_lcd_writeControl(0x04 + 0x02 + 0x01);	// Scroll to left
	vgl_lcd_writeControl(0x04 + 0x00);
	
	// CLS
	vgl_lcd_writeControl(0x01);
	
	// Home
	vgl_lcd_writeControl(0x02);
	
	
	// First byte is missing if we do not delay enough
	vgl_delay_1fff();
	
	
	//vgl_lcd_clear();
	vgl_cursorX = 0;
	vgl_cursorY = 0;
	//vgl_lcd_update();
}




/*
void vgl_lcd_shift_x() {
	// bit 4: cursor or display shift (2=direction, 3=which)
	// +4 = to the left, else right
	// +8 = display, else cursor
	//vgl_lcd_writeControl(0x10 + 0*0x04 + 0*0x08);
	vgl_lcd_writeControl(0x10 + 1*0x04 + 1*0x08);	// 0x08=disp
}
*/


/*
// Scroll
di
	push de
	push hl
	push bc
	push af

	; Move everything up by one row
	ld	bc, (_screen_scrollSize)
	ld	hl, _LCD_VRAM_ROW1
	ld	de, _LCD_VRAM_ROW0
	ldir	; Copy BC chars from (HL) to (DE)
	
	
	; DE should already be at the right position
	ld	h, d
	ld	l, e
	
	;ld	hl, _LCD_VRAM_ROW0
	;ld	de, (_screen_scrollSize)
	;add	hl, de
	
	; HL is now at the beginning of the last row
	
	ld	bc, (_screen_cols)		; Number of chars
	xor c
	
	ld	a, 0x20					; Which char to use: SPACE
	
	; Set B bytes at HL
	_screen_scroll_clear_loop:
		ld	(hl), a
		inc	hl
	djnz _screen_scroll_clear_loop
	
	pop af
	pop bc
	pop hl
	pop de
ei
*/
void vgl_lcd_scroll() {
	//vgl_lcd_writeControl(0x01);	// Clear
	//vgl_lcd_writeControl(0x02);	// Home
	//memset(LCD_VRAM, 0, LCD_COLS*LCD_ROWS);
	byte x, y;
	byte *cUp = LCD_VRAM_ROW0;
	byte *cDown = LCD_VRAM_ROW1;
	for (y = 0; y < LCD_ROWS-1; y++) {
		for (x = 0; x < LCD_COLS; x++) {
			*cUp = *cDown;
			cUp++;
			cDown++;
		}
		LCD_REFRESH_ROWS[y] = 1;
	}
	for (x = 0; x < LCD_COLS; x++) {
		*cUp++ = 0x20;
	}
	LCD_REFRESH_ROWS[LCD_ROWS-1] = 1;
}

void vgl_lcd_clear() {
	
	//vgl_lcd_writeControl(0x01);	// Clear
	//vgl_lcd_writeControl(0x02);	// Home
	//memset(LCD_VRAM, 0, LCD_COLS*LCD_ROWS);
	
	byte x, y;
	byte *c = LCD_VRAM;
	for (y = 0; y < LCD_ROWS; y++) {
		for (x = 0; x < LCD_COLS; x++) {
			*c++ = 0x20;
		}
		LCD_REFRESH_ROWS[y] = 1;
	}
	
	vgl_cursorX = 0;
	vgl_cursorY = 0;
	//vgl_lcd_update();
}


void vgl_lcd_update() {
	
	while (vgl_cursorX >= LCD_COLS) {
		vgl_cursorX -= LCD_COLS;
		vgl_cursorY++;
	}
	
	while (vgl_cursorY >= LCD_ROWS) {
		// Clear
		//vgl_cursorY = 0;
		//vgl_lcd_clear();
		
		// Scroll
		vgl_lcd_scroll();
		vgl_cursorY--;
	}
	
	// Update cursor
	//vgl_lcd_writeControl(0x08 + 0x04 + 0x02 + 0x01);
	
	LCD_CURSOR_MODE = 1+2+4;
	LCD_CURSOR_COL = vgl_cursorX;
	LCD_CURSOR_ROW = vgl_cursorY;
}

void vgl_lcd_putchar(char c) {
	// Either like this:
	//vgl_lcd_writeData(c);
	//return;
	
	// Or directly to RAM
	LCD_VRAM[(vgl_cursorY * LCD_COLS) + vgl_cursorX] = c;
	LCD_REFRESH_ROWS[vgl_cursorY] = 1;
	
	
	vgl_cursorX++;
	vgl_lcd_update();
	
}
#endif //__VGL_LCD_H