#!/bin/sh

NAME=app_busBuddy2

LOC_CODE=0x0000
LOC_DATA=0xc000

OUT_DIR=out
LIB_DIR=`realpath ../includes`
INC_DIR=${LIB_DIR}

INPUT_FILE=${NAME}.c
OUTPUT_FILE_HEX=${OUT_DIR}/${NAME}.hex
OUTPUT_FILE_BIN64K=${OUT_DIR}/${NAME}.64k.bin
OUTPUT_FILE_BIN=${OUT_DIR}/${NAME}.bin
OUTPUT_FILE_APP_BASE=${OUT_DIR}/${NAME}

CRT_S_FILE=${LIB_DIR}/vgl_crt0.s
CRT_REL_FILE=${OUT_DIR}/vgl_crt0.rel

if [ -f ${OUTPUT_FILE_HEX} ]; then
  rm ${OUTPUT_FILE_HEX}
fi
if [ -f ${OUTPUT_FILE_BIN} ]; then
  rm ${OUTPUT_FILE_BIN}
fi

# --code-loc 0x8000
# --stack-loc 0x20
# --data-loc 0x30
# --idata-loc 0x80
# --xram-loc 0xc000
# --model-small
# --no-std-crt0
# --nostdlib
# --verbose

# Using default crt0
#sdcc -mz80 --vc --code-loc 0x8000 -o ${OUTPUT_FILE_HEX} ${INPUT_FILE}

## Using a custom crt0
#echo Compiling CRT "${CRT_S_FILE}"...
#sdasz80 -o ${CRT_REL_FILE} ${CRT_S_FILE}

echo Compiling "${INPUT_FILE}"...
# "code-loc" specifies where the C code whould reside. Put it past the CRT code which begins at 0x8000
#sdcc -mz80 --no-std-crt0 --lib-path ${LIB_DIR} -I ${INC_DIR} --vc --code-loc 0x8030 --data-loc 0xc000 -o ${OUTPUT_FILE_HEX} ${CRT_REL_FILE} ${INPUT_FILE}
#sdcc -mz80 --no-std-crt0 --lib-path ${LIB_DIR} -I ${INC_DIR} --vc --code-loc 0x8030 --data-loc 0xc000 -o ${OUTPUT_FILE_HEX} ${CRT_REL_FILE} ${INPUT_FILE}

# Loadable apps without CRT
sdcc -mz80 --no-std-crt0 --nostdlib --lib-path ${LIB_DIR} -I ${INC_DIR} --vc --code-loc ${LOC_CODE} --data-loc ${LOC_DATA} -o ${OUTPUT_FILE_HEX} ${INPUT_FILE}

RETVAL=$?
[ $RETVAL -eq 0 ] && echo SDCC compile: Success
[ $RETVAL -ne 0 ] && echo SDCC compile: Failure ${RETVAL}



# Convert .hex to .bin
if [ -f ${OUTPUT_FILE_HEX} ]; then
  #echo Building loadable ROM "${OUTPUT_FILE_BIN}"...
  #makebin ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN}
  echo ----------------------------------------
  echo Building raw app binary "${OUTPUT_FILE_BIN}"...
  makebin -p -s 65536 ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN64K}
  # Extract a 16kB ROM cartridge binary (4096 x 4 bytes)
  #dd bs=4096 count=4 skip=32768 iflag=skip_bytes if=${OUTPUT_FILE_BIN64K} of=${OUTPUT_FILE_BIN}
  dd bs=4096 count=4 skip=0 iflag=skip_bytes if=${OUTPUT_FILE_BIN64K} of=${OUTPUT_FILE_BIN}
  
  #objcopy -Iihex -Obinary ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN}
  #objcopy -Iihex -Obinary --set-start 0x8000 ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN}
  
  #echo ----------------------------------------
  #echo Disassembling...
  ###z80dasm --origin=32768 --address --labels --source out/${NAME}.bin
  #z80dasm --origin=0 --address --labels --source out/${NAME}.bin
  
  echo ----------------------------------------
  echo Creating relocatable app...
  python rel2app.py ${OUTPUT_FILE_APP_BASE}
  
  echo ----------------------------------------
  echo Uploading app binary via serial loader...
  #python serloader_uploader.py -n -i ${OUTPUT_FILE_APP_BASE}.app.bin
  python serloader_uploader.py --input=${OUTPUT_FILE_APP_BASE}.app.bin
  
  RETVAL=$?
  echo Serial test run returned ${RETVAL}

  echo ----------------------------------------
  echo Done.
fi