/*

SPI Test
========

-> WORK IN PROGRESS AND SLIM CHANCES OF EVER WORKING!

All the hussle of getting re-locatable apps and serial loader working was done for this.
I am trying to create a "bitbanged" SPI port through the parallel port of the "V-Tech Genius Leader 4000 Quadro" the same way I achieved bitbanged 9600 serial.

MAME emulation can only get you so far - when it comes to actual timing, I need the real hardware.
And re-burning an EEPROM for every code modification seems wasteful.

2019-07-16 Bernhard "HotKey" Slawik

*/

#include <app.h>	// Load all app defines (to re-use the loader's API)


/*

TODO:
	* Test what happens if not 0xff, but a different mask is sent to port[0x11]
	* Change vgl_softserial to use ONE pin as TX (see port[0x10] and maybe bitmask on port[0x11])
	* Just communicate with an Arduino...



Connections:

	SPI 	dir	Parallel 	Bit
	----	---	---------	-------
	MISO	-->	11 (BUSY)	in[0x11] 0x5f / 0x7f
	SCLK	<--	 1 (STROBE)	out[0x12] ^ 0x04
	MOSI	<--	 4 (D2)  	out[0x10] ^ 0x04
	SS  	<--	 5 (D3)  	out[0x10] ^ 0x08
	.... 	<--	 6 (D4)  	out[0x10] ^ 0x10
	
BEWARE to use a 5V-to-3.3V level shifter for MOSI, SCLK and SS!

*/

#define SPI_MASK_MISO 0x5f	// on port 0x11
#define SPI_MASK_SCLK 0x04	// on port 0x12
#define SPI_MASK_MOSI 0x04	// on port 0x10
#define SPI_MASK_SS 0x08	// on port 0x10


byte spi_get_bool() __naked {
__asm
	in	a, (0x11)	; Get printer status
	cp	#0x5f
	jr	nz, _spi_get_bit_no

_spi_get_bit_yes:
	ld	l, #1
	ret

_spi_get_bit_no:
	ld	l, #0
	ret
__endasm;
}

byte spi_get_value() __naked {
__asm
	in	a, (0x11)	; Get printer status
	ld	l, a
	ret
__endasm;
}

void inline spi_set_data(byte c) {
	(void)c;	// Suppress warning "unreferenced"
	__asm
		out	(0x10), a	; Set printer DATA
	__endasm;
}

void inline spi_push_data() {
	__asm
		ld	a, #0xff			; Send all data bits to the pins
		out	(0x11), a
	__endasm;
}

void inline spi_clock_low() {
	__asm
		in	a, (0x12)			; Can be omitted if A still valid
		or	#0x04				; STROBE LOW...
		out	(0x12), a
	__endasm;
}

void inline spi_clock_high() {
	__asm
		in	a, (0x12)			; Can be omitted if A still valid
		and	#0xfb				; STROBE HIGH...
		out	(0x12), a
	__endasm;
}



void spi_cycle(byte c) {
	(void)c;	// Suppress warning "unreferenced"
	
	__asm
	
		out	(0x10), a			; Set printer DATA
		
		ld	a, #0xff			; Send all data bits to the pins
		out	(0x11), a
		
		in	a, (0x12)
		or	#0x04				; STROBE LOW...
		out	(0x12), a
		
		
		and	#0xfb				; STROBE HIGH... (takes a >50us to reach HIGH)
		out	(0x12), a
	
	__endasm;
}

void spi_test1() {
	byte i;
	
	spi_cycle(1*SPI_MASK_SS);
	
	for(i = 0; i < 8; i++) {
		spi_cycle(1*SPI_MASK_SS + (i%2)*SPI_MASK_MOSI);
	}
	
	spi_cycle(0*SPI_MASK_SS);
	
}

void spi_test2() {
	byte i;
	
	spi_clock_low();
	
	for(i = 0; i < 8; i++) {
		spi_set_data(1*SPI_MASK_MOSI);
		spi_push_data();
		
		spi_set_data(0*SPI_MASK_MOSI);
		spi_push_data();
	}
	
	spi_clock_high();
	
}

int main(int argc, char *argv[]) {
	char c;
	
	(void)argc;	(void)argv;	// Mark args as "not used" to suppress warnings
	
	
	printf("Hello SPI world!\n");
	
	
	do {
		printf("1,2,0Push,CLk,Quit\n");
		
		c = getchar();
		
		switch(c) {
			case '1':
				spi_test1();
				break;
			
			case '2':
				spi_test2();
				break;
			
			case '0':
				spi_set_data(0);
				break;
			case 'p':
				spi_push_data();
				break;
			
			case 'c':
				spi_clock_low();
				spi_clock_high();
				break;
			case 'l':
				spi_clock_low();
				break;
			case 'h':
				spi_clock_high();
				break;
			
			default:
				spi_set_data(c);
		}
		
	} while(c != 'q');
	
	
	return 0;
}
