/*

Serial Loader
=============

App loader for V-Tech Genius Leader notebooks.
This program uses "loader.h" to load "apps", link (!) and run them.

Programs are received via bitbanged serial port.

2019-07-13 Bernhard "HotKey" Slawik

*/

#include <vgl.h>	// Hardware drivers

#include <stdio.h>
#include <string.h>

#include "loader.h"	// Loader architecture (including API)
#include <hex.h>	// hex dump helpers

#include <vgl_softserial.h>

// Include external app(s)
//#include "out/app_hello.app.h"

// Desired destination addresses
const word app_destCode = 0xc800;
const word app_destData = 0xc400;
const word tempAddr = 0xc400;
#define MAX_DATA 0x4000

// Some test arguments
#define MAX_ARGS 8
const char *testArgs[MAX_ARGS];	// = {(char *)testArg0, (char *)testArg1};

#define MAX_LINE 255	// Maximum length of one incoming line

void idle() {
	word t;
	for(t = 0; t < 0x1000; t++) {
		// Give the CPU a few cycles for interrupts
		;
	}
}

void signal_fault() {
	// Slow low pitch sound
}
void signal_ok() {
	// Short high pitch sound
}

/*
void serial_mon_0x10() {
	while(1) {
		printf("%02X", port_in_0x10());
	}
}
void serial_mon_0x11() {
	while(1) {
		printf("%02X", port_in_0x11());
	}
}
void serial_mon_0x12() {
	while(1) {
		printf("%02X", port_in_0x12());
	}
}

void serial_rx() {
	int c;
	printf("rx:");
	while(1) {
		c = serial_getchar();
		if (c >= 0) {
			putchar(c);
		}
	}
}

void serial_rxHex() {
	int c;
	printf("rxHex:");
	while(1) {
		c = serial_getchar();
		if (c >= 0) {
			//putchar(c);
			printf("%d %04X\n", c, c);
		}
	}
}
void serial_loadBin() {
	int r;
	word l;
	byte *buf;
	int c;
	word timeout;
	
	
	buf = (byte *)tempAddr;	// Where to load data to
	l = 0;
	
	
	printf("Waiting for data");
	// Loop
	c = -1;
	do {
		
		// Send "ready" signal
		printf(".");
		serial_put("#READY\n");
		
		// Wait for data (with timeout)
		timeout = 0x4000;
		do {
			c = serial_getchar();
			timeout--;
		} while((c < 0) && (timeout > 0));
	} while(c < 0);
	
	// Okay, there is data!
	//printf("!");
	
	timeout = 0x1000;
	//while ((c >= 0) && (l < MAX_DATA)) {
	while (timeout > 0) {
		*buf++ = c;
		timeout = 0x1000;
		do {
			c = serial_getchar();
			timeout--;
		} while((c < 0) && (timeout > 0));
	}
	
	
	l = (word)buf - tempAddr;
	printf("%d bytes RXed to %04X.\n", l, tempAddr);
	serial_put("#OK\n");
	beep();
	
	//dump(tempAddr, 8);
	
	printf("Press key to link");
	getchar();
	printf("\n");
	
	app_load((byte *)tempAddr, app_destCode, app_destData);
	
	// Load another app from a different memory region
	//clear();
	//printf("app_hello:\n");
	//app_load(APP_HELLO, app_destCode, app_destData);
	
	printf("Press key to start");
	getchar();
	printf("\n");
	
	//app_callStart();	// Call start() function
	r = app_callMain(2, testArgs);	// Call main() function
	
	
	beep();
	printf("Exit code: %d\n", r);
	getchar();
	
	serial_put("#END\n");
}
*/
void serial_loadHex() {
	char line[MAX_LINE];
	byte *buf;
	int r;
	word a;
	word aCode;
	word aData;
	byte b;
	byte i;
	word l;
	byte n;
	char *c;
	
	
	//buf = (byte *)tempAddr;	// Where to load data to
	printf("Waiting for data...\n");
	
	idle();
	serial_put("#RDY\n");
	
	while(1) {
		serial_gets(&line[0]);
		l = strlen(&line[0]);
		
		// Search for "#"
		c = &line[0];
		while ((*c != '#') && (l > 0)) {
			c++;
			l--;
		}
		if (l <= 0) continue;
		
		if ((l > (4+2+4)) && (strncmp(c, "#PUT", 4) == 0)) {
			c += 4;
			n = hextown(c, 2); c += 2;
			if (l < (4+2+4+2*n)) {
				// Too short!
				signal_fault();
				serial_put("#ETS\n");
				continue;
			}
			a = hextown(c, 4); c += 4;
			
			// Disallow writing to zero-page
			if (a == 0x0000) {
				signal_fault();
				serial_put("#EA0\n");
				continue;
			}
			
			printf("%d byte @ %04X...", n, a);
			
			//buf = (byte *)(tempAddr + a);	// Destination address
			buf = (byte *)(a);	// Destination address
			for(i = 0; i < n; i++) {
				b = hextown(c, 2); c += 2;
				*buf++ = b;
			}
			printf("OK\n");
			signal_ok();
			serial_put("#POK\n");
		} else
		if ((l >= (4+2+4)) && (strncmp(c, "#GET", 4) == 0)) {
			c += 4;
			n = hextown(c, 2); c += 2;
			a = hextown(c, 4); c += 4;
			//buf = (byte *)(tempAddr + a);	// Source address
			buf = (byte *)a;	// Source address
			
			c = &line[0];
			sprintf(&line[0], "#DMP%04X", a); c += 8;
			c = &line[8];
			for(i = 0; i < n; i++) {
				b = *buf++;
				//sprintf(c, "%02X", b); c += 2;
				*c++ = hexDigit(b >> 4);
				*c++ = hexDigit(b & 0x0f);
			}
			*c++ = 0x0a;
			*c++ = 0;
			serial_put(&line[0]);
			
		} else
		if ((l >= (4)) && (strncmp(c, "#END", 4) == 0)) {
			c += 4;
			break;
			
		} else
		if ((l >= (4+4)) && (strncmp(c, "#RUN", 4) == 0)) {
			// Link and run APP binary at given address
			c += 4;
			//serial_put("#ROK\n");
			a = hextown(c, 4); c += 4;
			
			// Optional CODE and DATA destination addresses
			if (l >= (4+4+4))	{ aCode = hextown(c, 4); c += 4; }
			else				aCode = app_destCode;
			
			if (l >= (4+4+4+4))	{ aData = hextown(c, 4); c += 4; }
			else				aData = app_destData;
			
			printf("Reloc. %04X to %04X");
			app_load((byte *)a, aCode, aData);
			
			
			sprintf(&line[0], "#ROK%04X\n", a);
			serial_put(&line[0]);
			
			printf("Calling main()...\n");
			beep();
			
			testArgs[0] = "Hello";
			testArgs[1] = "Hello";
			r = app_callMain(2, testArgs);	// Call main() function
			
			beep();
			printf("Exit code: %d\n", r);
			//getchar();
			
			sprintf(&line[0], "#RET%04X\n", r);
			serial_put(&line[0]);
		
		} else {
			//printf("\"%s\"?\n", line);
			serial_put("#E??\n");
		}
	}
	
	serial_put("#END\n");
}

// Main cartridge entry point
void main() {
	//char c;
	
	// Init VGL hardware
	vgl_sound_off();
	vgl_lcd_init();
	vgl_lcd_clear();
	vgl_lcd_update();
	
	// Banner
	printf("Serial Loader\n");
	idle();
	serial_put("#HLO\n");
	
	serial_loadHex();
	
	/*
	c = 0;
	while(c != 'q') {
		//printf("Press key to recv.");
		printf("012=MON L=ldHEX B=ldBIN R=RX H=HEX");
		c = getchar();
		switch(c) {
			case '0': serial_mon_0x10(); break;
			case '1': serial_mon_0x11(); break;
			case '2': serial_mon_0x12(); break;
			case 'b': serial_loadBin(); break;
			case 'l': serial_loadHex(); break;
			case 'r': serial_rx(); break;
			case 'h': serial_rxHex(); break;
		}
	}
	*/
	
	printf("End of loader. STOP.");
	while(1) {}
	
}
