#!/bin/sh

NAME=memview

OUT_DIR=out
LIB_DIR=`realpath ../includes`
INC_DIR=${LIB_DIR}

INPUT_FILE=${NAME}.c
OUTPUT_FILE_HEX=${OUT_DIR}/${NAME}.hex
OUTPUT_FILE_BIN64K=${OUT_DIR}/${NAME}.64k.bin
OUTPUT_FILE_BIN=${OUT_DIR}/${NAME}.bin

CRT_S_FILE=${LIB_DIR}/vgl_crt0.s
CRT_REL_FILE=${OUT_DIR}/vgl_crt0.rel

if [ -f ${OUTPUT_FILE_HEX} ]; then
  rm ${OUTPUT_FILE_HEX}
fi
if [ -f ${OUTPUT_FILE_BIN} ]; then
  rm ${OUTPUT_FILE_BIN}
fi

# --code-loc 0x8000
# --stack-loc 0x20
# --data-loc 0x30
# --idata-loc 0x80
# --xram-loc 0xc000
# --model-small
# --no-std-crt0
# --nostdlib
# --verbose

# Using default crt0
#sdcc -mz80 --vc --code-loc 0x8000 -o ${OUTPUT_FILE_HEX} ${INPUT_FILE}

# Using a custom crt0
echo Compiling CRT "${CRT_S_FILE}"...
sdasz80 -o ${CRT_REL_FILE} ${CRT_S_FILE}

echo Compiling "${INPUT_FILE}"...
# "code-loc" specifies where the C code whould reside. Put it past the CRT code which begins at 0x8000
#sdcc -mz80 --no-std-crt0 --lib-path ${LIB_DIR} -I ${INC_DIR} --vc --code-loc 0x8030 --data-loc 0xc000 -o ${OUTPUT_FILE_HEX} ${CRT_REL_FILE} ${INPUT_FILE}
sdcc -mz80 --no-std-crt0 --lib-path ${LIB_DIR} -I ${INC_DIR} --vc --code-loc 0x8030 --data-loc 0xc000 -o ${OUTPUT_FILE_HEX} ${CRT_REL_FILE} ${INPUT_FILE}

# Convert .hex to .bin
if [ -f ${OUTPUT_FILE_HEX} ]; then
  echo Building usable cartridge ROM "${OUTPUT_FILE_BIN}"...
  makebin -p -s 65536 ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN64K}
  dd bs=4096 skip=32768 iflag=skip_bytes if=${OUTPUT_FILE_BIN64K} of=${OUTPUT_FILE_BIN}
  
  #objcopy -Iihex -Obinary ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN}
  #objcopy -Iihex -Obinary --set-start 0x8000 ${OUTPUT_FILE_HEX} ${OUTPUT_FILE_BIN}
  echo Ready.
fi