;; crt0 for a Z80 based V-Tech Genius Leader / PreComputer
;; Based on SDCC's z80 crt0 and stuff I have implemented for z88dk's "vgl" target

;; SDCC asm syntax: https://github.com/darconeous/sdcc/blob/master/sdas/doc/asmlnk.txt
;; vgl info: /z/data/_code/_c/z88dk.git/libsrc/_DEVELOPMENT/target/vgl

	.module crt0
	.globl	_main

	.area	_HEADER (ABS)
	
;; Cartridge ROM start
	.org 0x8000

	;; ROM signature
	.db	#0x55
	.db	#0xaa

	;; Auto-start signature for ROM cartridges (if found: will start instantly on power-on)
	.db	#0x59	; "Y"
	.db	#0x45	; "E"

	; "GA" = normal signature (i.e. non-autostart code)
	;defb 47h ; "G"
	;defb 41h ; "A"

	; PreComputer1000 autostart
	;.db #0x33  ; 0x33 = autostart jump to 0x8010
	;.db #0x00  ; Dont care

;; First executed instruction (usually "jp 0x801f")
; Jump to actual code
jp	init

;; The following bytes can be used for custom ROM header information

;; ROM info
;	rom_text:
;		.ascii /VGL_CRT0 by HotKey 2019/
;	
;	rom_version:
;		.db #0x01


;; Some useful low-level stuff

;	;PUBLIC _print_char
;	.globl _putchar
;	_putchar:
;		; Get parameter from stack into a
;		ld hl,#0x0002
;		add hl,sp
;		ld a,(hl)
;		
;		; Put it to port (just a proof-of concept)
;		out	(0x0b), a
;		call _delay_1fff
;		
;		ret
;	; End of useful stuff, back to init


;; Init
; The first instruction bytes jump to this label
;	.org	0x8020

init:
	di ; disable interrupts
	
	;; Set stack pointer directly above top of memory.
	ld	sp, #0xdff0	; Load StackPointer to 0xdff0
	
	;; Initialise global variables
	;call	gsinit
	
	;; Initialise hardware
	;call vgl_sound_off
	;call vgl_lcd_init
	
	ei
	
	;; System is ready
	
	;; Call main() function (C entry point)
	call	_main
	
	;; Jump to shutdown code
	jp	_exit

	;; Ordering of segments for the linker.
	.area	_HOME
	.area	_CODE
	.area	_INITIALIZER
	.area   _GSINIT
	.area   _GSFINAL
	
	.area	_DATA
	.area	_INITIALIZED
	.area	_BSEG
	.area   _BSS
	.area   _HEAP
	
	.area   _CODE
	
__clock::
	ld	a,#2
	rst	0x08
	ret

_exit::
	;call 0x0164 ; V-Tech 4000 Shutdown animation
	
	;; Exit - special code to the emulator
	;ld	a,#0
	;rst	0x08
	
	; V-Tech
	;	ld	a, 1
	;	out	(0ah), a
	;	
	;	ld	a, 1
	;	out	(12h), a
	;	halt
	
1$:
	halt
	jr	1$

	.area   _GSINIT
gsinit::
;	ld	bc, #l__INITIALIZER
;	ld	a, b
;	or	a, c
;	jr	Z, gsinit_next
;	ld	de, #s__INITIALIZED
;	ld	hl, #s__INITIALIZER
;	ldir
gsinit_next:

;	.area   _GSFINAL
;	ret


